# @justia/babel-preset

> A babel preset for transforming your JavaScript for Airbnb.
> Up-to-date based on `babel-preset-airbnb@4.4.0`

Currently contains transforms for all [stage 4](https://tc39.github.io/ecma262/) (ES2018) and [stage 3](https://github.com/tc39/proposals#active-proposals) syntax that is permitted in the _[Justia Style Guide]_. Please note that if usage of a stage 3 proposal is not explicitly mentioned in the Guide, then it will not be enabled here. Additionally, stage 4 syntax that is excluded is as follows:

- generators: `regenerator-runtime` is too heavyweight for our use.
- `async/await`: `regenerator-runtime` is too heavyweight for our use, and [async-to-promises](https://www.npmjs.com/package/babel-plugin-async-to-promises) is not yet complete enough to be safely used.
- async iterators: depends on both generators and `async function`s
- lifted template literal restrictions: we do not use tagged template literals, nor implement custom DSLs, otherwise we would enable this.

## Install

Install the correct versions of the peer dependencies, which are listed by the command:

```shell
npm info "@justia/babel-preset@latest" peerDependencies
```

If using **npm 5+**, use this shortcut

```shell
npx install-peerdeps --dev --auth <token> @justia/babel-preset
```

Where **`<token>`** is a **read-only [npm authentication token](https://docs.npmjs.com/creating-and-viewing-authentication-tokens)** to allow `install-peerdeps` to inspect and get the correct versions of the packages.

> Recommendation:
>
> Create a new environment variable in your profile (`~/.bash_profile`, `~/.zshrc`, `~/.profile`, or `~/.bashrc`) to hold the token and replace `<token>` with **`$NPM_TOKEN`**

If for some reason you could not create the [npm token](https://docs.npmjs.com/about-authentication-tokens) (it is not that hard, try again, seriously `ヽ(ಠ▃ಠ)ﾉ`), you can run

```shell
(
export PKG=@justia/babel-preset;
npm info "$PKG@latest" peerDependencies --json | command sed 's/[\{\},]//g ; s/: /@/g' | xargs npm install --save-dev "$PKG@latest"
)
```

Which produces and runs a command like:

```shell
npm install --save-dev @justia/babel-preset @babel/core@^#.#.# @babel/runtime@^#.#.#
```

## Usage

### Via `.babelrc` (Recommended)

**.babelrc**

```json
{
  "presets": [
    "@justia/babel-preset"
  ]
}
```

### Via CLI

```shell
babel script.js --presets @justia/babel-preset
```

### Via Node API

```javascript
require('@babel/core').transform('code', {
    presets: ['@justia/babel-preset']
});
```

<!-- No "Targeting Environments" -->

## React Development Mode

When `process.env.NODE_ENV` is `'development'`, [the `development` mode will be set for `@babel/preset-react`](https://babeljs.io/docs/en/babel-preset-react#development).

You may override our default development option by providing your own boolean `development` key.

```json
{
  "presets": [["@justia/babel-preset", {
    "development": false
  }]]
}
```

<!-- No "React PropTypes removal" -->

## Classes loose mode

By default, this preset will compile classes in normal mode. This is safer, but comes with a bundle size and runtime overhead. To [compile classes in loose mode](https://babeljs.io/docs/en/babel-plugin-transform-classes#loose), set the `looseClasses` option to `true`:

```json
{
  "presets": [["@justia/babel-preset", {
    "looseClasses": true,
  }]]
}
```

The [risks of enabling loose classes are outlined in the Babel docs](https://babeljs.io/docs/en/babel-plugin-transform-classes#loose).

## Specifying a babel runtime version

By default `@babel/plugin-transform-runtime` will [assume the oldest version of the runtime](https://github.com/babel/babel/blob/e6264a09921c60b8f18870d0a75678e4fa04f0f8/packages/babel-plugin-transform-runtime/src/index.js#L42) to avoid importing helpers that don't exist which would fail at runtime. This can result in newer helpers being inlined into modules (ex. objectSpread2) which increases bundle size.

To avoid this you can configure the preset to use the same version of the runtime that's installed in your package.json.

ex. If package.json has `"@babel/runtime": "^7.7.4"` then you can use:

```json
{
  "presets": [["@justia/babel-preset", {
    "runtimeVersion": "7.7.4",
  }]]
}
```

Note that this will result in a runtime breakage if the version passed into the airbnb preset is newer than the version of the babel runtime actually being used at build time.

## Disabling `plugin-transform-runtime`

You can use the `transformRuntime` option to disable [`@babel/plugin-transform-runtime`](https://babeljs.io/docs/en/babel-plugin-transform-runtime). Specifying `false` will disable the plugin. This option defaults to `true`.

## Specifying module transforms

You can use the `modules` option to enable transformation of modules given to this preset:

```json
{
  "presets": [["@justia/babel-preset", {
    "modules": "auto"
  }]]
}
```

Both `true` and the option default `auto` will not transform modules if ES6 module syntax is already supported by the environment, or `"commonjs"` otherwise. `false` will not transform modules.

You can use the `runtimeHelpersUseESModules` option to prevent transformation of runtime helpers to CommonJS modules.

```json
{
  "presets": [["@justia/babel-preset", {
    "runtimeHelpersUseESModules": true
  }]]
}
```

`true` will not transform runtime helpers to CommonJS modules. `false` will transform runtime helpers to CommonJS modules. The option defaults to `true` if `modules` is set to `false`, and `false` otherwise.

## Configure `preset-env`

### `useBuiltIns`

Use `useBuiltIns` to configure how `@babel/preset-env` handles polyfills.

```json
{
  "presets": [["@justia/babel-preset", {
    "useBuiltIns": "usage"
  }]]
}
```

When either the `usage` or `entry` options are used, `@babel-preset-env` will add direct references to `core-js` modules as bare imports (or requires). This means `core-js` will be resolved relative to the file itself and needs to be accessible. This option defaults to `false`.

Since `@babel/polyfill` was deprecated in `7.4.0`, we recommend directly adding `core-js` and setting the version via the [corejs](#corejs) option.

```shell
npm install core-js@3 --save

# or

npm install core-js@2 --save
```

Read more info about the specific details of each option in the [`@babel/preset-env` official page](https://babeljs.io/docs/en/babel-preset-env#usebuiltins).

### `corejs`

This option only has an effect when used alongside `useBuiltIns: usage` or `useBuiltIns: entry`, and ensures `@babel/preset-env` injects the correct imports for your `core-js` version. The valid values for this are `2`, `3` or `{ version: 2 | 3, proposals: boolean }`, defaults to `2`.

By default, only polyfills for stable ECMAScript features are injected: if you want to polyfill them, you have three different options:

- when using `useBuiltIns: "entry"`, you can directly import a [proposal polyfill](https://github.com/zloirock/core-js/tree/master/packages/core-js/proposals): `import "core-js/proposals/string-replace-all"`.
- when using `useBuiltIns: "usage"` you have two different alternatives:
    - set the [`shippedProposals`](https://babeljs.io/docs/en/babel-preset-env#shippedproposals) option to `true`. This will enable polyfills and transforms for proposal which have already been shipped in browsers for a while.
use corejs: `{ version: 3, proposals: true }`. This will enable polyfilling of every proposal supported by `core-js`.


```json
{
  "presets": [["@justia/babel-preset", {
    "corejs": 3
  }]]
}
```
